import { EntityRepository, getConnection, getManager, Repository } from "typeorm";
import { CreateBlogPostDto } from "./dto/create-blog-post.dto";
import { UpdateBlogPostDto } from "./dto/update-blog-post.dto";
import { BlogPost } from "./entities/blog-post.entity";

@EntityRepository(BlogPost)
export class BlogPostRepository extends Repository<BlogPost> {

    async createEntity(dto: CreateBlogPostDto): Promise<any | Error> {
        try {
            const entity = this.create({
                image: dto.image,
                name: dto.name,
                author: dto.author,
                publishDate: dto.publishDate,
                urlSlug: dto.urlSlug,
                description: dto.description
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findAll(): Promise<BlogPost[] | Error> {
        try {
            return await this.find({
                order: {
                    createdAt: -1
                },
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<BlogPost[] | Error> {
        try {
            return await this.find({
                where: {
                    id: id
                }
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntityBySlug(urlSlug: string): Promise<BlogPost[] | Error> {
        try {
            return await this.find({
                where: {
                    urlSlug: urlSlug
                }
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(BlogPost)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async updateBlogPost(id: string, dto: UpdateBlogPostDto): Promise<BlogPost | Error> {
        try {
            await this.update(id, {
                image: dto.image,
                name: dto.name,
                author: dto.author,
                publishDate: dto.publishDate,
                urlSlug: dto.urlSlug,
                description: dto.description
            });
            return await this.findOne({ where: { id: id } });
        } catch (e) {
            return e;
        }
    }

    async getLatestBlogList(): Promise<BlogPost[] | Error> {
        try {
            return await this.find({
                where: {
                    isDeleted: false
                },
                order: { 
                    createdAt: "DESC" 
                },
                take: 6
            });
        }
        catch (e) {
            return e;
        }
    }

    async countAllBlogs(): Promise<any | Error> {
        try {
            return await this.count({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async findBlogByFilterQuery(
        page: number,
        limit: number
    ): Promise<any | Error> {
        try {
            const count = await this.find({
                where: {
                    isDeleted: false
                }
            });
            const posts = await this.find({
                where: {
                    isDeleted: false
                },
                take: limit,
                skip: limit * (page - 1)
            });
            const pageCount = Math.ceil(count.length / limit);
            return {
                page: page,
                limit: limit,
                pageCount: pageCount,
                posts
            };
        } catch (e) {
            return e;
        }
    }


    async paginate(page: number, limit: number): Promise<any | Error> {
        try {
            let offset = 0;

            if (page > 1) {
                offset = (page - 1) * limit;
            }

            const blogSQL = `SELECT * FROM blog_posts 
            WHERE isDeleted = 0 ORDER BY createdAt ASC LIMIT ${limit} OFFSET ${offset};`;

            const blogCountSQL = `SELECT count(*) as count FROM blog_posts WHERE isDeleted = 0;`;

            const blogListCount = await this.manager.query(blogCountSQL);
            const blogList = await this.manager.query(blogSQL);

            const totalPages = Math.ceil(blogListCount?.[0]?.count / limit);
            const nextPage = page < totalPages ? page + 1 : null;
            const previousPage = page > 1 ? page - 1 : null;

            return {
                blogs: blogList,
                totalCount: blogListCount[0].count,
                totalPages,
                nextPage,
                previousPage
            }
        } catch (e) {
            return e;
        }
    }
}